from django.views.generic.base import TemplateView
from example.app_bar import BarConfig
from site_config.backends.model_backend import models

class HomeView(TemplateView):
    
    def get_context_data(self, **kwargs):
        kwargs['websites'] = models.Website.objects.all()
        return kwargs
        


from __future__ import unicode_literals
from django.contrib import admin
from django import forms
from django.core.urlresolvers import reverse
from site_config import registry
from . import models
from . import forms as backend_forms


class WebsiteApplicationInline(admin.TabularInline):
    def detail_link(self, obj):
        return "<a href='%s'>detail</a>" % (reverse('admin:site_config_websiteapplication_change', args=(obj.id,)))
    detail_link.short_description = 'Link to Detail Page'
    detail_link.allow_tags = True
    
    model = models.WebsiteApplication
    exclude = ['options']
    readonly_fields = ['detail_link']
    extra = 0


class WebsiteAdmin(admin.ModelAdmin):
    #inlines = [WebsiteApplicationInline,]
    list_display=['id', 'name', 'short_name', 'active', ]
    list_editable = ['active',]
    list_filter=['active']
    prepopulated_fields = {"short_name": ("name",)}
    ordering = ['short_name']


class ApplicationAdminForm(forms.ModelForm):
    
    def __init__(self, *args, **kwargs):
        super(ApplicationAdminForm, self).__init__(*args, **kwargs)
        self.fields['short_name'] = forms.ChoiceField(choices=registry.config_registry.get_config_list())
    
    class Meta:
        model = models.Application


class ApplicationAdmin(admin.ModelAdmin):
    form = ApplicationAdminForm
    #inlines = [WebsiteApplicationInline,]    
    list_display=['id', 'short_name', 'active',]
    list_editable=['active',]
    list_filter=['active']
    ordering = ['short_name']


class WebsiteApplicationAdmin(admin.ModelAdmin):

    def application_active(self, obj):
        return True if obj.application.active else False
    application_active.short_description = 'App Active'
    application_active.boolean = True

    def website_active(self, obj):
        return True if obj.website.active else False
    website_active.short_description = 'Website Active'
    website_active.boolean = True

    def website_link(self, obj):
        return_value = None
        if obj:
            return_value =  "<a href='%s' onclick='return showAddAnotherPopup(this);'>Link</a>" % (
                reverse('admin:site_config_website_change', args=(obj.website.id,)))
        return return_value
    website_link.short_description = 'Link'
    website_link.allow_tags = True

    def application_link(self, obj):
        return_value = None
        if obj:
            return_value =  "<a href='%s' onclick='return showAddAnotherPopup(this);'>Link</a>" % (
                reverse('admin:site_config_application_change', args=(obj.application.id,)))
        return return_value
    application_link.short_description = 'Link'
    application_link.allow_tags = True

    list_display=['id', 'website', 'website_active', 'application', 'application_active', 'active',]
    list_editable=['active',]
    list_filter=['active', 'website', 'application']
    readonly_fields = ['website_active', 'application_active', 'website_link', 'application_link']

    def get_form(self, request, obj=None, **kwargs):
        self.fieldsets = [
            [None, {
                'fields': (('website', 'website_active', 'website_link'), 
                           ('application', 'application_active', 'application_link',),
                           'active', 'curtain_message', 'description', )
            }],
        ]

        form = backend_forms.website_application_formfactory(instance=obj)
        if form.config_fields:
            self.fieldsets.append(["Configuration Options", {"fields":['reset_options',] + form.config_fields}],)
        return form

    def save_model(self, request, obj, form, change):
        if obj.id is None:
            config_lookup = registry.config_registry.get_config_class(
                obj.application.short_name
            )
            default_config = config_lookup[1]().get_default_configs()
            obj.set_config_options(default_config, save=False)
        else:
            obj.set_config_options(
                form.cleaned_data.get('options', {}), save=False)
        obj.save()


admin.site.register(models.Website, WebsiteAdmin)
admin.site.register(models.Application, ApplicationAdmin)
admin.site.register(models.WebsiteApplication, WebsiteApplicationAdmin)




from __future__ import unicode_literals
import logging
import inspect

from django.http import Http404
from django.http import HttpResponseNotFound
from django.template.loader import render_to_string
from . import choices, utils

logger = logging.getLogger(__name__)

###### Decorator Utilities


def decorated_includes(wrapping_functions, patterns_rslt):
    '''
    SOURCE: http://stackoverflow.com/questions/2307926/is-it-possible-to-decorate-include-in-django-urls-with-login-required
    
    Used to require 1..n decorators in any view returned by a url tree

    Usage:
      urlpatterns = decorated_includes(func,patterns(...))
      urlpatterns = decorated_includes((func,func,func),patterns(...))

    Note:
      Use functools.partial to pass keyword params to the required 
      decorators. If you need to pass args you will have to write a 
      wrapper function.

    Example:
      from functools import partial

      urlpatterns = decorated_includes(
          partial(login_required,login_url='/accounts/login/'),
          patterns(...)
      )
    '''
    if not hasattr(wrapping_functions,'__iter__'): 
        wrapping_functions = (wrapping_functions,)

    return [
        _wrap_instance__resolve(wrapping_functions, instance)
        for instance in patterns_rslt
    ]

def _wrap_instance__resolve(wrapping_functions, instance):
    if not hasattr(instance,'resolve'): return instance
    resolve = getattr(instance,'resolve')

    def _wrap_func_in_returned_resolver_match(*args,**kwargs):
        rslt = resolve(*args,**kwargs)
        

        if not hasattr(rslt,'func'):return rslt
        f = getattr(rslt,'func')

        for _f in reversed(wrapping_functions):
            # @decorate the function from inner to outter
            f = _f(f)

        setattr(rslt,'func',f)

        return rslt

    setattr(instance,'resolve',_wrap_func_in_returned_resolver_match)

    return instance



###### Decorator Functions

def enable_disable_website(the_func, config_class):
    """
    This determines whether or not the view should be
    considered active, curtained, or disabled (404)
    for the webapp based on the value of    
    website_application_status() from the backend.
    
    This takes a view function and a config class.
    The view function should contain a 'website' 
    keyword argument, either as part of the url regex 
    or passed in via the url()'s kwargs dictionary.
    
    If the webapp is active, the view is processed as normal.
    If the webapp is curtained, a 404 is returned and a
    curtain message is printed to the screen (as defined
    by the get_curtain_message() backend method.)
    If the webapp is disabled, a 404 is returned. 
    
    """

    def _decorated(*args, **kwargs):
        website = kwargs.get('website', None)
        app_config = config_class(website=website)
        active_state = app_config.website_application_status()
        if active_state == choices.WEBAPP_ACTIVE_STATE_CURTAINED:
            if args[0].user.is_authenticated() and args[0].user.is_superuser:
                pass
            else:
                logger.debug("Website Application curtained %s - %s" % (website, app_config.application_short_name))
                return HttpResponseNotFound(render_to_string("site_config/curtained.html", 
                    {'message':'%s' % app_config.get_curtain_message()}))
        elif active_state == choices.WEBAPP_ACTIVE_STATE_ENABLED:
            pass
        else:
            logger.debug("Website Application disabled %s - %s" % (website, app_config.application_short_name))
            raise Http404
        return the_func(*args, **kwargs)
    return _decorated


def website_template_override(the_func, template_kwarg_name="template_name"):

    def _decorated(*args, **kwargs):
        website = kwargs.get('website', None)
        # Check url pattern for template_name definition
        template_name = kwargs.get(template_kwarg_name, None)
        # Check function kwarg for template_name definition
        # Also check to make sure that the_func is a function
        #   i.e. django.contrib.syndication.views.Feed is a 
        #   valid input to a url, but is not a function 
        if website and not template_name and inspect.isfunction(the_func):
            _args, varargs, varkw, defaults = inspect.getargspec(the_func)
            result = {}
            if defaults:
                firstdefault = len(_args) - len(defaults)
            for i, arg in enumerate(_args):
                if defaults and i >= firstdefault:
                    result[arg] = defaults[i - firstdefault]
            if template_kwarg_name in result:
                template_name = result['template_name']
        if website and template_name:
            template = utils.website_override_template(template_name, website)
            kwargs.update({template_kwarg_name:template.name})
        return the_func(*args, **kwargs)
    return _decorated




from django.conf.urls import patterns, include, url
from django.contrib import admin
from site_config.decorators import enable_disable_website, website_template_override, decorated_includes
from example.app_foo import FooConfig
from example.app_bar import BarConfig
from . import views

admin.autodiscover()


urlpatterns = patterns('',
    url(r'^$', views.HomeView.as_view(template_name='index.html'), {}, name="index"),                       
    url(r'^admin/', include(admin.site.urls)),
)

urlpatterns += decorated_includes(lambda func: enable_disable_website(func, FooConfig), 
    patterns('', url(r'^(?P<website>\w+)/foo/', include('example.app_foo.urls')))
)

urlpatterns += decorated_includes(lambda func: enable_disable_website(func, BarConfig),
    patterns('', url(r'^(?P<website>\w+)/bar/', include('example.app_bar.urls')))
)




from django.views.generic.base import TemplateView
from django.shortcuts import render_to_response
from django.template import RequestContext
from site_config.utils import WebsiteOverrideTemplateViewMixin
from site_config.decorators import website_template_override
from example.app_bar import BarConfig

class IndexView(WebsiteOverrideTemplateViewMixin, TemplateView):
    
    def dispatch(self, request, *args, **kwargs):
        self.website = kwargs.get('website', None)
        self.config = BarConfig(website=self.website)
        return super(IndexView, self).dispatch(request, *args, **kwargs)
    
    def get_context_data(self, **kwargs):
        kwargs['config'] = self.config
        kwargs['website'] = self.website
        return kwargs


@website_template_override
def index(request, template_name, website=None, *args, **kwargs):
    config = BarConfig(website=website)
    return render_to_response(template_name,
                              {
                               'website':website,
                               'config':config,
                               'template_name':template_name,
                               },
                              context_instance=RequestContext(request))
    
    



WEBAPP_ACTIVE_STATE_DISABLED="disabled"
WEBAPP_ACTIVE_STATE_CURTAINED="curtained"
WEBAPP_ACTIVE_STATE_ENABLED="enabled"

WEBAPP_ACTIVE_STATES = (
  (WEBAPP_ACTIVE_STATE_DISABLED, "Disabled"),
  (WEBAPP_ACTIVE_STATE_CURTAINED, "Curtained"),
  (WEBAPP_ACTIVE_STATE_ENABLED, "Enabled"),
)


from __future__ import unicode_literals
import logging
from site_config import utils, choices
from .. import ConfigBackend
from . import models

logger = logging.getLogger(__name__)

class DatabaseBackend(ConfigBackend):

    def get(self, key, config_dict, application_short_name, website_short_name=None):
        return self.mget(config_dict, application_short_name, website_short_name).get(key)
    
    def mget(self, config_dict, application_short_name, website_short_name=None):
        # set a default 'value' in each nested config dict
        config_dict = utils.config_dict_value_from_default(config_dict)
        # lookup the site application
        site_app_list = models.WebsiteApplication.objects.website_applications(
                        website_short_name=website_short_name, 
                        application_short_name=application_short_name, 
                        )
        if site_app_list.count() == 1:
            site_app = site_app_list[0]
            config_dict = site_app.get_config_options(config_dict)
        return config_dict
    
    def set(self, config_name, value, config_dict, application_short_name, website_short_name=None):
        config_dict = config_dict.copy()
        if config_name in config_dict:
            config_dict[config_name].update({'value':value})
        return self.mset(config_dict, application_short_name, website_short_name)
    
    def mset(self, config_dict, application_short_name, website_short_name=None):
        try:
            site_app = models.WebsiteApplication.objects.get(
                                application__short_name=application_short_name, 
                                website__short_name=website_short_name)
            site_app.set_config_options(config_dict)
        except models.WebsiteApplication.DoesNotExist:
            raise 
        return site_app
    
    def website_application_status(self, application_short_name, website_short_name):
        active = choices.WEBAPP_ACTIVE_STATE_DISABLED
        site_app_list = models.WebsiteApplication.objects.website_applications(
                        website_short_name=website_short_name, 
                        application_short_name=application_short_name, 
                        )
        if site_app_list.count() == 1:
            site_app = site_app_list[0]
            active = site_app.active_status()
        return active

    def get_curtain_message(self, application_short_name, website_short_name=None):
        message = ("This site is undergoing scheduled maintenance." 
                   "Thank you for your patience.")
        site_app_list = models.WebsiteApplication.objects.website_applications(
                        website_short_name=website_short_name, 
                        application_short_name=application_short_name, 
                        )
        if site_app_list.count() == 1:
            site_app = site_app_list[0]
            message_from_model = site_app.get_curtain_message()
            if message_from_model:
                message = message_from_model
        return message


from __future__ import unicode_literals
from site_config.utils import website_override_template
from site_config.backends.model_backend.models import Website
from django.conf import settings



def decide_base_template(request):
    base_name = getattr(
        settings, 'SITECONFIG_BASE_TEMPLATE', 'base_site.html'
    )    
    website_obj = None
    try:
        website = request.resolver_match.kwargs.get('website', None)
    except:
        website = None

    if website:
        base_name = website_override_template(base_name, website).name
    else:        
        parts = [part for part in request.path.split('/') if part != ""]
        if len(parts) > 0:
            website = parts[0]
            base_name = website_override_template(base_name, website).name

    if website:
        try:
            website_obj = Website.objects.get(short_name=website)
        except Website.DoesNotExist as e:
            website_obj = None

    return {
        'base_template': base_name,
        'website_shortname': website,
        'website': website_obj
    }



from django import forms
import site_config

class FooConfig(site_config.SiteConfigBase):

    application_short_name = "foo"
    application_verbose_name = "Foo App"

    def get_default_configs(self):
        return {'FOO_EMAIL':{'default':"joe@exampe.com",
                             'field':'django.forms.EmailField', 
                             'help':'Test A help text.'}, 
                "FOO_NUM":{'default':1, 
                           'field':'django.forms.IntegerField', 
                           'help':'Test B help text.'}}

site_config.registry.config_registry.register(FooConfig)



from __future__ import unicode_literals
import mock
from copy import deepcopy
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase
from django.test.utils import override_settings
from site_config import utils
from . import lib


@override_settings(**lib.settings_overrides)
class TestUtils(lib.SiteConfigMixin, TestCase):
    def setUp(self):
        self.load_config()

    def test_config_dict_value_from_default__value_key_added(self):
        self.assertNotIn("value", self.config_dict['TEST_A'])
        self.assertNotIn("value", self.config_dict['TEST_B'])
        updated_config_dict = utils.config_dict_value_from_default(self.config_dict)
        self.assertIn("value", updated_config_dict['TEST_A'])
        self.assertIn("value", updated_config_dict['TEST_B'])
        self.assertNotIn("value", self.config_dict['TEST_A'])
        self.assertNotIn("value", self.config_dict['TEST_B'])

    def test_config_dict_value_from_default__default_equals_value(self):
        for k, v in utils.config_dict_value_from_default(self.config_dict).items():
            self.assertEqual(v['default'], v['value'], )

    def test_update_config_dict__single_update(self):
        updated_config_dict = {'TEST_A': {"value": "5432"},
                               }
        updated = utils.update_config_dict(self.config_dict, updated_config_dict)
        self.assertEqual(updated["TEST_A"]["value"], "5432", )
        self.assertFalse(updated["TEST_B"].has_key("value"))

    def test_update_config_dict__all_update(self):
        updated_config_dict = {'TEST_A': {"value": "5432"},
                               'TEST_B': {"value": 2},
                               }
        updated = utils.update_config_dict(self.config_dict, updated_config_dict)
        self.assertEqual(updated["TEST_A"]["value"], "5432", )
        self.assertEqual(updated["TEST_B"]["value"], 2, )

    def test_update_config_dict__extra_update(self):
        updated_config_dict = {'TEST_A': {"value": "5432"},
                               'TEST_C': {"value": 2},
                               }
        updated = utils.update_config_dict(self.config_dict, updated_config_dict)
        self.assertEqual(updated["TEST_A"]["value"], "5432", )
        self.assertTrue(updated.has_key("TEST_B"))
        self.assertFalse(updated.has_key('TEST_C'))

    @mock.patch('site_config.utils.select_template')
    def test_website_override_tamplate__exist(self, select_template):
        select_template.return_value = 'Joe/app1'
        self.assertEqual('Joe/app1',
            utils.website_override_template('Joe', 'app1')
                         )

    @mock.patch('site_config.utils.select_template')
    def test_website_override_tamplate__unexist(self, select_template):
        select_template.return_value = 'app1'
        self.assertEqual('app1',
            utils.website_override_template('Joe', 'app1')
                         )

    class baseObj(object):
        def __init__(self):
            self.template_name = 'app.html'
            self.website = 'Example'
            self.kwargs = {}

        def get_template_names(self):
            # Direct implementation from TemplateResponseMixin
            if self.template_name is None:
                raise ImproperlyConfigured(
                    "TemplateResponseMixin requires either a definition of "
                    "'template_name' or an implementation of "
                    "'get_template_names()'"
                )
            else:
                return [self.template_name]

    class testObj(utils.WebsiteOverrideTemplateViewMixin, baseObj):
        pass

    def test_website_override_template_view_mixin_website_set(self):
        test_obj = self.testObj()
        self.assertEqual(
            set(test_obj.get_template_names()),
            set(['Example/app.html', 'app.html'])
        )

    def test_website_override_template_view_mixin_website_set_kwargs_set(self):
        test_obj = self.testObj()
        test_obj.kwargs['website'] = 'Dan'
        self.assertEqual(
            set(test_obj.get_template_names()),
            set(['Example/app.html', 'app.html'])
        )

    def test_website_override_template_view_mixin_no_website_no_kwargs(self):
        test_obj = self.testObj()
        test_obj.website = None
        self.assertEqual(
            set(test_obj.get_template_names()),
            set(['app.html'])
        )

    def test_website_override_template_view_mixin_no_website_set_kwargs(self):
        test_obj = self.testObj()
        test_obj.website = None
        test_obj.kwargs['website'] = 'Different'
        self.assertEqual(
            set(test_obj.get_template_names()),
            set(['Different/app.html','app.html'])
        )




import mock
# from copy import deepcopy
# from django.conf.urls import patterns, url, include
from django.http import Http404
from django.test import TestCase
from django.test.utils import override_settings
from site_config import decorators
from .test_modelbackend_basic_usage import ModelsBaiscMixin
from . import lib


@override_settings(**lib.settings_overrides)
class TestDecorators(ModelsBaiscMixin, lib.SiteConfigMixin, TestCase):
    def helper_func(*args, **kwargs):
        if 'template_name' in kwargs.keys() and kwargs['template_name'] == []:
            return False
        return True

    class User:
        def __init__(self, authed, superuser):
            self.authed = authed
            self.is_superuser = superuser

        def is_authenticated(self):
            return self.authed

    def setUp(self):
        self.load_config()
        self.load_models()
        self.site_config.registry.config_registry.register(self.MyAppSiteConfig)

    def _set_model_active_state(self, website, application, webapp):
        self.Website.objects.filter().update(active=website)
        self.Application.objects.filter().update(active=application)
        self.WebsiteApplication.objects.filter().update(active=webapp)

    def test_enable_disable_website__enabled(self):
        self._set_model_active_state(True, True, "enabled")
        decorated_func = decorators.enable_disable_website(self.helper_func, self.MyAppSiteConfig)
        context = {'website': self.site1_short_name}
        self.assertTrue(decorated_func(**context))

    def test_enable_disable_website__disabled(self):
        self._set_model_active_state(True, True, "disabled")
        decorated_func = decorators.enable_disable_website(self.helper_func, self.MyAppSiteConfig)
        context = {'website': self.site1_short_name}
        try:
            decorated_func(**context)
        except Http404:
            pass

    def test_enable_disable_website__curtained_not_superuser(self):
        self._set_model_active_state(True, True, "curtained")
        decorated_func = decorators.enable_disable_website(self.helper_func, self.MyAppSiteConfig)
        context = {'website': self.site1_short_name}
        mock_user = mock.Mock()
        mock_user.user = self.User(False, False)
        args = [mock_user, ]
        self.assertTrue('maintenance' in decorated_func(*args, **context).content)

    def test_enable_disable_website__curtained_is_superuser_authed(self):
        self._set_model_active_state(True, True, "curtained")
        decorated_func = decorators.enable_disable_website(self.helper_func, self.MyAppSiteConfig)
        context = {'website': self.site1_short_name}
        mock_user = mock.Mock()
        mock_user.user = self.User(True, True)
        args = [mock_user, ]
        self.assertTrue(decorated_func(*args, **context))

    def test_enable_disable_website__curtained_is_superuser_unauthed(self):
        self._set_model_active_state(True, True, "curtained")
        decorated_func = decorators.enable_disable_website(self.helper_func, self.MyAppSiteConfig)
        context = {'website': self.site1_short_name}
        mock_user = mock.Mock()
        mock_user.user = self.User(False, True)
        args = [mock_user, ]
        self.assertTrue('maintenance' in decorated_func(*args, **context).content)

    def test_enable_disable_website__curtained_not_superuser_authed(self):
        self._set_model_active_state(True, True, "curtained")
        decorated_func = decorators.enable_disable_website(self.helper_func, self.MyAppSiteConfig)
        context = {'website': self.site1_short_name}
        mock_user = mock.Mock()
        mock_user.user = self.User(True, False)
        args = [mock_user, ]
        self.assertTrue('maintenance' in decorated_func(*args, **context).content)


def func(request, website=None, template_name='app.html'):
    return template_name


def func_no_template_name(request, website=None):
    return


class PretentCBV(object):
    def __call__(self, request, website):
        pass


@override_settings(**lib.settings_overrides)
class TestTemplateOverride(ModelsBaiscMixin, lib.SiteConfigMixin, TestCase):

    @mock.patch('site_config.utils.select_template')
    def test_template_name_from_url_context(self, select_template_mock):
        """
        template_name from URL context should have priority
        """
        decorated_func = decorators.website_template_override(func)
        context = {'website': 'site1', 'template_name': 'app2.html'}
        decorated_func(None, **context)
        select_template_mock.assert_called_once_with(
            ('site1/app2.html', 'app2.html')
        )

    @mock.patch('site_config.utils.select_template')
    def test_template_name_from_function_defaults(self, select_template_mock):
        """
        template_name from func kwargs should be used if available
        """
        decorated_func = decorators.website_template_override(func)
        context = {'website': 'site1'}
        decorated_func(None, **context)
        select_template_mock.assert_called_once_with(
            ('site1/app.html', 'app.html')
        )

    @mock.patch('site_config.utils.select_template')
    def test_no_website_kwarg_no_select_template_call(self, select_template_mock):
        """
        No select_template calls should be made if website not in context
        """
        decorated_func = decorators.website_template_override(func)
        context = {}
        decorated_func(None, **context)
        select_template_mock.assert_not_called()

    @mock.patch('site_config.utils.select_template')
    def test_no_template_name_func_acceptance(self, select_template_mock):
        """
        No select_template calls should be made if website not in context
        """
        decorated_func = decorators.website_template_override(func)
        context = {'website': 'site1'}
        decorated_func(None, **context)
        select_template_mock.assert_not_called()

    @mock.patch('site_config.utils.select_template')
    def test_no_template_name_feed(self, select_template_mock):
        """
        No select_template calls should be made if function is a callable
        class instance, such as django Feed()
        """
        decorated_func = decorators.website_template_override(PretentCBV())
        context = {'website': 'site1'}
        decorated_func(None, **context)
        select_template_mock.assert_not_called()




settings_overrides = dict(
    INSTALLED_APPS=(
        'django.contrib.admin',
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.messages',
        'django.contrib.staticfiles',
        'site_config',
        'site_config.backends.model_backend',
    ),
    SITECONFIG_BACKEND_DEFAULT='site_config.backends.model_backend.DatabaseBackend',
)


class SiteConfigMixin(object):

    def load_config(self):

        config_dict = {'TEST_A': {'default':
                                 "Test A default",
                                 'field': 'django.forms.CharField',
                                 'help': 'Test A help text.'},
                       "TEST_B": {'default': 1,
                                 'field': 'django.forms.IntegerField',
                                 'help': 'Test B help text.'}}
        self.config_dict = config_dict

        self.site_config = __import__('site_config')

        class MyAppSiteConfig(self.site_config.SiteConfigBase):
            application_short_name = "myapp"
            application_verbose_name = "My Application"

            def get_default_configs(self):
                return config_dict
        self.MyAppSiteConfig = MyAppSiteConfig



from __future__ import unicode_literals
import logging
from jsonfield import JSONField
from django.utils.encoding import python_2_unicode_compatible
from django.db import models
from django.db.models.signals import post_save
from site_config import utils, choices

logger = logging.getLogger(__name__)

@python_2_unicode_compatible
class Website(models.Model):
    
    name = models.CharField(max_length=64, 
        help_text="Enter a descriptive name for this website.")
    short_name = models.SlugField(max_length=64, unique=True, 
        help_text="This must be a unique name using only "
                  "letter, numbers, hyphens, and underscores.")
    active = models.BooleanField(default=False, 
        help_text="Enable or disable the entire website.")
    description = models.TextField(blank=True, null=True)

    class Meta:
        app_label = 'site_config'
        verbose_name = "Website"
        verbose_name_plural = "Websites"
        ordering = ['short_name']

    def __str__(self):
        return "%s (%s)" % (self.name, self.short_name)


@python_2_unicode_compatible
class Application(models.Model):

    short_name = models.SlugField(max_length=64, unique=True,
        help_text="This must be a unique name using only "
                  "letter, numbers, hyphens, and underscores.")
    active = models.BooleanField(default=False, 
        help_text="Enable or disable the entire application.")
    description = models.TextField(blank=True, null=True)

    class Meta:
        app_label = 'site_config'
        verbose_name = "Application"
        verbose_name_plural = "Applications"
        ordering = ['short_name']

    def __str__(self):
        return "%s" % (self.short_name)


class WebsiteApplicationQuerySet(models.query.QuerySet):
    
    def active(self):
        return self.filter(active=True)

    def website_applications(self, website_short_name, application_short_name):
        return self.filter(
                        application__short_name=application_short_name, 
                        website__short_name=website_short_name, )


class WebsiteApplicationManager(models.Manager):
    
    def get_queryset(self):
        return WebsiteApplicationQuerySet(self.model, using=self._db)
    
    def __getattr__(self, attr, *args):
        try:
            return getattr(self.__class__, attr, *args)
        except AttributeError:
            return getattr(self.get_query_set(), attr, *args)
        

@python_2_unicode_compatible
class WebsiteApplication(models.Model):
    
    website = models.ForeignKey('site_config.Website')
    application = models.ForeignKey('site_config.Application')
    
    active = models.CharField(max_length=20,
        default=choices.WEBAPP_ACTIVE_STATE_DISABLED,
        choices=choices.WEBAPP_ACTIVE_STATES,
        help_text="Activates, curtains or deactivates this website application "
        "combination. In order for this to be active, both the corresponding "
        "website and application must also be active.  Curtained sites can "
        "only be viewed by superusers.")
    description = models.TextField(blank=True, null=True)
    curtain_message = models.TextField(blank=True, null=True, 
        default="This site is undergoing scheduled maintenance." 
                "Thank you for your patience.",
        help_text="Specify a message that should be displayed "
                   "when the site is in the curtained state.")
    options = JSONField(blank=True, null=True)
    
    objects = WebsiteApplicationManager()
    
    def get_config_options(self, default_config_dict):
        return utils.update_config_dict(default_config_dict, self.options)

    def set_config_options(self, config_dict, save=True):
        self.options = config_dict
        if save:
            self.save()

    def active_status(self):
        return_value = choices.WEBAPP_ACTIVE_STATE_DISABLED
        if self.website.active and self.application.active and \
           self.active in [choices.WEBAPP_ACTIVE_STATE_ENABLED, 
                           choices.WEBAPP_ACTIVE_STATE_CURTAINED]:
            return_value = self.active
        return return_value
    
    def get_curtain_message(self):
        return self.curtain_message

    class Meta:
        app_label = 'site_config'
        verbose_name = "Website Application"
        verbose_name_plural = "Websites Applications"
        unique_together = (('website', 'application'),)

    def __str__(self):
        return "%s (%s)" % (self.website, self.application)



from django.conf.urls import patterns, include, url
from site_config.decorators import enable_disable_website
from .views import IndexView

urlpatterns = [
   url('^$', IndexView.as_view(template_name='app_foo/index.html'), {}, name="app_foo_index")
]



import unittest

import mock

from site_config.backends.model_backend import admin, models


class TestWAAdmin(unittest.TestCase):
    @mock.patch('site_config.backends.model_backend.admin.registry')
    def test_save_model_no_id(self, registry_mock):
        """
        Assert default config used if obj is new (has no id)
        """
        wa_admin = admin.WebsiteApplicationAdmin(
            models.WebsiteApplication,
            mock.MagicMock()
        )
        obj = mock.Mock()
        obj.application.short_name = 'test'
        obj.id = None

        config = mock.MagicMock()
        config.return_value.get_default_configs.return_value = {'key': 'value'}
        config_list = ("Test", config)
        registry_mock.config_registry.get_config_class.return_value = (
            config_list
        )

        wa_admin.save_model(None, obj, None, None)

        registry_mock.config_registry.get_config_class.assert_called_with(
            'test'
        )
        obj.set_config_options.assert_called_with(
            {'key': 'value'},
            save=False
        )
        obj.save.assert_called()

    @mock.patch('site_config.backends.model_backend.admin.registry')
    def test_save_model_with_id(self, registry_mock):
        """
        Assert config returned from form is used if obj is not new (has id)
        """
        wa_admin = admin.WebsiteApplicationAdmin(
            models.WebsiteApplication,
            mock.MagicMock()
        )
        obj = mock.Mock()
        obj.id = 1

        form = mock.Mock()
        form.cleaned_data.get.return_value = {'key': 'val2'}

        wa_admin.save_model(None, obj, form, None)

        obj.set_config_options.assert_called_with({'key': 'val2'}, save=False)

        obj.save.assert_called()



from __future__ import unicode_literals
# import mock
from copy import deepcopy
from django.test import TestCase
from django.test.utils import override_settings
# from site_config import utils
from . import lib


settings = deepcopy(lib.settings_overrides)
settings.update(dict(SITECONFIG_BACKEND_DEFAULT="site_config.backends.settings_backend.SettingsBackend"))


@override_settings(**settings)
class TestSiteConfigRegistry(lib.SiteConfigMixin, TestCase):
    def setUp(self):
        self.load_config()

    def test_register_site_config(self):
        self.site_config.registry.config_registry.register(self.MyAppSiteConfig)
        self.assertIsNotNone(self.site_config.registry.config_registry.get_config_class('myapp'), )

    def test_register_site_config_verbose_name(self):
        self.site_config.registry.config_registry.register(self.MyAppSiteConfig)
        self.assertEqual(self.site_config.registry.config_registry.get_config_class('myapp')[0],
                         self.MyAppSiteConfig.application_verbose_name)

    def test_register_site_config_short_name(self):
        self.site_config.registry.config_registry.register(self.MyAppSiteConfig)
        config_class = self.site_config.registry.config_registry.get_config_class('myapp')[1]
        self.assertEqual(config_class.application_short_name,
                         self.MyAppSiteConfig.application_short_name)

    '''
    def test_config_list(self):
        self.site_config.registry.config_registry.register(self.MyAppSiteConfig)
        self.assertEqual(
            self.site_config.registry.config_registry.get_config_list(),
            dir(self.MyAppSiteConfig.application_short_name)
        )
    '''



# import mock
from copy import deepcopy
from django.test import TestCase
from django.test.utils import override_settings
# from site_config import settings as site_settings
from . import lib


settings = deepcopy(lib.settings_overrides)
settings.update(dict(
    SITECONFIG_BACKEND_DEFAULT="site_config.backends.settings_backend.SettingsBackend"),
    TEST_B='Result in B',
)


@override_settings(**settings)
class TestSettingsBackendBasicAccess(lib.SiteConfigMixin, TestCase):

    def setUp(self):
        self.load_config()
        self.site_config.registry.config_registry.register(self.MyAppSiteConfig)

    def test_attributeA_access(self):
        siteconfig = self.MyAppSiteConfig(website=None)
        self.assertEqual(siteconfig.TEST_A, "Test A default")

    def test_attributeB_access(self):
        siteconfig = self.MyAppSiteConfig(website=None)
        self.assertEqual(siteconfig.TEST_B, 'Result in B')

    def test_get_config_default_value_A(self):
        siteconfig = self.MyAppSiteConfig(website=None)
        self.assertEqual(siteconfig.get_config("TEST_A")['value'],
                         self.config_dict['TEST_A']['default'])

    def test_get_config_has_proper_keys_A(self):
        siteconfig = self.MyAppSiteConfig(website=None)
        self.assertEqual(self.config_dict["TEST_A"].keys() + ["value"],
                        siteconfig.get_config("TEST_A").keys(), )

    def test_get_config_has_proper_keys_B(self):
        siteconfig = self.MyAppSiteConfig(website=None)
        self.assertEqual(self.config_dict["TEST_B"].keys() + ['value'],
                        siteconfig.get_config("TEST_B").keys(), )

    def test_get_config_default_value_B(self):
        siteconfig = self.MyAppSiteConfig(website=None)
        self.assertEqual(siteconfig.get_config("TEST_B")['value'],
                         self.config_dict['TEST_B']['value'])


@override_settings(**settings)
class TestSettingsConfigActive(lib.SiteConfigMixin, TestCase):

    def _set_setting_active_state(self, application, webapp):
        self.Application.objects.filter().update(active=application)
        self.WebsiteApplication.objects.filter().update(active=webapp)

    def setUp(self):
        self.load_config()
        self.site_config.registry.config_registry.register(self.MyAppSiteConfig)





from django import forms
import site_config

class BarConfig(site_config.SiteConfigBase):

    application_short_name = "bar"
    application_verbose_name = "Bar App"

    def get_default_configs(self):
        return {'BAR_EMAIL':{'default':"joe@exampe.com", 
                               'field':'django.forms.CharField', 
                               'help':'Test A help text.'}, 
                "BAR_NUM":{'default':1, 
                             'field':'django.forms.IntegerField', 
                             'help':'Enter a number.'},
                "BAR_CHAR":{'default':"b", 
                              'field':'django.forms.ChoiceField', 
                              'help':'Test B help text.', 
                              'choices':(('a',"A choice"),("b","B Choice"),)},
                }

site_config.registry.config_registry.register(BarConfig)


#!/usr/bin/env python
import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(name='site_config',
      version='0.0.14',
      description='Django configuration Utility to manage multiple "websites" in a project. ',
      long_description=read('README.md'),
      author='Joe Jasinski',
      author_email='jjasinski@imagescape.com',
      install_requires=[
       'django >= 1.6', 
       'jsonfield >= 0.9.22',
      ],
      url='https://github.com/ImaginaryLandscape/django_site_config',
      keywords='django site config configuration',
      extras_require = {
        'testing':  ["mock >= 1.0.1", ],
      },
      packages=find_packages(),
      include_package_data=True,
      classifiers=[
        "Framework :: Django", 
        "Topic :: Utilities",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: BSD License",
      ],
)



from django.conf.urls import patterns, include, url
from site_config.decorators import enable_disable_website
from .views import IndexView

urlpatterns = [
   url('^page3/$', 'example.app_bar.views.index', {'template_name':'app_bar/index.html'}, name="app_bar_page3"),  
   url('^page2/$', 'example.app_bar.views.index', {'template_name':'app_bar/index.html'}, name="app_bar_page2"),  
   url('^page1/$', IndexView.as_view(template_name='app_bar/index.html'), {}, name="app_bar_page1"),   
   url('^$', IndexView.as_view(template_name='app_bar/index.html'), {}, name="app_bar_index"),
]



from django.views.generic.base import TemplateView
from site_config.utils import WebsiteOverrideTemplateViewMixin
from example.app_foo import FooConfig

class IndexView(WebsiteOverrideTemplateViewMixin, TemplateView):
    
    def dispatch(self, request, *args, **kwargs):
        self.website = kwargs.get('website', None)
        self.config = FooConfig(website=self.website)
        return super(IndexView, self).dispatch(request, *args, **kwargs)
    
    def get_context_data(self, **kwargs):
        kwargs['config'] = self.config
        kwargs['website'] = self.website
        return kwargs
        


# import mock
from copy import deepcopy
from django.test import TestCase
from django.test.utils import override_settings
from . import lib


class ModelsBaiscMixin(object):

    def load_models(self):
        self.Website = self.site_config.backends.model_backend.models.Website
        self.Application = self.site_config.backends.model_backend.models.Application
        self.WebsiteApplication = self.site_config.backends.model_backend.models.WebsiteApplication

        self.sites = []
        self.site1_short_name = "joe"
        site = self.Website(
            name="JOE SITE", short_name=self.site1_short_name,
            active=True)
        site.save()
        self.sites.append(site)

        self.site2_short_name = "john"
        site = self.Website(
            name="John Site", short_name=self.site2_short_name,
            active=False)
        site.save()
        self.sites.append(site)

        self.apps = []
        self.app1_short_name = 'myapp'
        app = self.Application(short_name=self.app1_short_name, active=True)
        app.save()
        self.apps.append(app)

        webapp = self.WebsiteApplication(
            website=self.sites[0], application=self.apps[0],
            active="enabled")
        webapp.save()
        self.webapps = []
        self.webapps.append(webapp)


settings = deepcopy(lib.settings_overrides)
settings.update(dict(SITECONFIG_BACKEND_DEFAULT="site_config.backends.model_backend.DatabaseBackend"))


@override_settings(**settings)
class TestModelBackendBasicAccess(ModelsBaiscMixin, lib.SiteConfigMixin, TestCase):

    def setUp(self):
        self.load_config()
        self.load_models()
        self.site_config.registry.config_registry.register(self.MyAppSiteConfig)

    def test_attributeA_access(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self.assertEqual(siteconfig.TEST_A, "Test A default")

    def test_attributeB_access(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self.assertEqual(siteconfig.TEST_B, 1)
        self.assertNotEqual(siteconfig.TEST_B, "1")

    def test_get_config_default_value_A(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self.assertEqual(siteconfig.get_config("TEST_A")['value'],
                         self.config_dict['TEST_A']['default'])

    def test_get_config_has_proper_keys_A(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self.assertEqual(self.config_dict["TEST_A"].keys() + ["value"],
                        siteconfig.get_config("TEST_A").keys(), )

    def test_get_config_has_proper_keys_B(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self.assertEqual(self.config_dict["TEST_B"].keys() + ['value'],
                        siteconfig.get_config("TEST_B").keys(), )

    def test_get_config_default_value_B(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self.assertEqual(siteconfig.get_config("TEST_B")['value'],
                         self.config_dict['TEST_B']['default'])

    '''
    def test_get_configs_default_value_A(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self.assertEqual(siteconfig.get_configs(), siteconfig.get_default_configs())
    '''

    def test_set_config_A_update(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        siteconfig.set_config("TEST_A", "JOE")
        self.assertEqual(siteconfig.get_config("TEST_A")['value'], "JOE")
        self.assertEqual(siteconfig.TEST_A, "JOE")

    def test_set_config_B_update(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        siteconfig.set_config("TEST_B", 666)
        self.assertEqual(siteconfig.get_config("TEST_B")['value'], 666)
        self.assertEqual(siteconfig.TEST_B, 666)

    def test_invalid_attribute(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self.assertFalse(hasattr(siteconfig, 'TEST_C'))


@override_settings(**settings)
class TestConfigActive(ModelsBaiscMixin, lib.SiteConfigMixin, TestCase):

    def _set_model_active_state(self, website, application, webapp):
        self.Website.objects.filter().update(active=website)
        self.Application.objects.filter().update(active=application)
        self.WebsiteApplication.objects.filter().update(active=webapp)

    def setUp(self):
        self.load_config()
        self.load_models()
        self.site_config.registry.config_registry.register(self.MyAppSiteConfig)

    def test_website_application_status__all_active(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self._set_model_active_state(True, True, "enabled")
        self.assertEqual(siteconfig.website_application_status(), "enabled")

    def test_website_application_status__incorrect_status(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self._set_model_active_state(True, True, "incorrect_status")
        self.assertEqual(siteconfig.website_application_status(), "disabled")

    def test_website_application_status__app_and_siteapp_active(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self._set_model_active_state(False, True, "enabled")
        self.assertEqual(siteconfig.website_application_status(), "disabled")

    def test_website_application_status__wesite_and_siteapp_active(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self._set_model_active_state(True, False, "enabled")
        self.assertEqual(siteconfig.website_application_status(), "disabled")

    def test_website_application_status__siteapp_active(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self._set_model_active_state(False, False, "enabled")
        self.assertEqual(siteconfig.website_application_status(), "disabled")

    def test_website_application_status__nothing_active(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self._set_model_active_state(False, False, "disabled")
        self.assertEqual(siteconfig.website_application_status(), "disabled")

    def test_get_curtain_message__nothing_active(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self._set_model_active_state(False, False, "curtained")
        self.assertTrue("This site is undergoing scheduled maintenance" in siteconfig.get_curtain_message())

    def test_get_curtain_message__website_active(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self._set_model_active_state(True, False, "curtained")
        self.assertTrue("This site is undergoing scheduled maintenance" in siteconfig.get_curtain_message())

    def test_get_curtain_message__website_and_siteapp__active(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self._set_model_active_state(True, True, "curtained")
        self.assertTrue("This site is undergoing scheduled maintenance" in siteconfig.get_curtain_message())

    def test_dir(self):
        siteconfig = self.MyAppSiteConfig(website=self.site1_short_name)
        self.assertEqual(set(siteconfig.__dir__()), set(['TEST_A', 'TEST_B']))
